<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once MOONY_ROOT_DIR . '/Moony/Action.php';
require_once MOONY_ROOT_DIR . '/Moony/Registry.php';
require_once MOONY_ROOT_DIR . '/Moony/Request.php';
require_once MOONY_ROOT_DIR . '/Moony/Router.php';
require_once MOONY_ROOT_DIR . '/Moony/Session.php';

/**
 * 処理を順番に行います。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Processor
{
    /**
     * 処理を以下の順番で行います。
     *   1. リクエストを司るコンポーネントの生成
     *   2. ルーティング処理（実行アクションの決定）
     *   3. アクションクラスのプロパティにリクエストパラメータを展開
     *   4. セッションを司るコンポーネントの生成、セッションの設定
     *   5. アクションクラスのinitメソッドを実行
     *   6. セッションの開始
     *   7. フラッシュ変数をアクションクラスのプロパティに展開
     *   8. アクションクラスのfilterメソッドを実行
     *   9. アクションクラスのvalidateメソッドを実行
     *  10. アクションクラスのexecuteメソッドを実行
     *
     * @access public
     * @param object $moony Moonyのインスタンス（動作設定）
     */
    function process(&$moony)
    {
        // request
        $request =& new Moony_Request();
        if (!ini_get('mbstring.encoding_translation') &&
            !is_null($moony->input_encoding)) {
            // convert encoding
            $request->convertEncoding(
                $moony->input_encoding, $moony->internal_encoding);
        }

        // routing
        $router =& new Moony_Router();
        $action = $router->route($request, $moony->action_dir);
        if ($action === false) {
            // not defined: action_dir
            trigger_error('Not defined: action_dir', E_USER_ERROR);
            exit;
        }
        if (is_null($action)) {
            // not found: action class
            $action =& $this->_createBlankAction();
        }

        // session
        $session =& new Moony_Session();
        if (is_array($moony->session_options)) {
            // set options
            $this->_setSessionOptions($session, $moony->session_options);
        }

        // set to the action instance
        $action->moony = &$moony;
        $action->request = &$request;
        $action->args = $this->_getActionArgs($request);
        $action->session = &$session;

        // save in the registry
        Moony_Registry::save('_moony_action', $action);

        // extract the request parameters
        $this->_setActionProperties($action, $request->getAll());

        // action::init
        if (method_exists($action, 'init')) {
            $action->init();
        }

        // start the session
        if ($moony->session_auto_start) {
            // start
            $session->start();
            // extract the flash values
            $flash_vars = $session->get('_moony_flash');
            if (is_array($flash_vars)) {
                $this->_setActionProperties($action, $flash_vars);
            }
            $session->set('_moony_flash', array());
        }

        ob_start();

        // action::filter
        if (method_exists($action, 'filter')) {
            include_once MOONY_ROOT_DIR . '/Moony/Filter.php';
            $filter =& new Moony_Filter($action->moony->internal_encoding);
            $action->filter($filter);
        }

        // action::validate
        $has_error = false;
        if (method_exists($action, 'validate')) {
            include_once MOONY_ROOT_DIR . '/Moony/Validator.php';
            $validator =&
                new Moony_Validator($action->moony->internal_encoding);
            $result = $action->validate($validator);
            if ($result === false) {
                $has_error = true;
                Moony_Registry::save('_moony_has_error', $has_error);
            } else {
                $has_error = $validator->hasError();
            }
        }

        // action::execute
        if ($has_error === false) {
            $action->execute();
        }

        ob_end_flush();
    }

    /**
     * Moony_Actionを継承しただけのアクションクラスを生成、
     * インスタンスを取得します。
     *
     * @access protected
     * @return object アクションクラスのインスタンス
     */
    function &_createBlankAction()
    {
        $code = 'class BlankAction extends Moony_Action {}';
        eval($code);
        $action =& new BlankAction();
        return $action;
    }

    /**
     * アクションにPATH_INFO経由で渡されたパラメータを取得します。
     *
     * @access protected
     * @param object $request Moony_Requestのインスタンス
     * @return array PATH_INFO経由で渡されたパラメータの配列
     */
    function _getActionArgs(&$request)
    {
        $path_info = $request->getPathInfo();
        $action_path = $request->getActionPath();
        $arg_str = substr(ltrim($path_info, '/'), strlen($action_path));
        $arg_str = ltrim($arg_str, '/');
        return strlen($arg_str) == 0 ? array() : explode('/', $arg_str);
    }

    /**
     * アクションクラスのプロパティを設定します。
     * 名称が「_」で始まるパラメータと、「moony」「request」
     * 「session」「args」という名称のパラメータは
     * 展開されません（Moonyの処理で使用するため）。
     *
     * @access protected
     * @param object $action アクションクラスのインスタンス
     * @param array $properties プロパティに設定する値の連想配列
     */
    function _setActionProperties(&$action, $properties)
    {
        static $ignore_keys = array('moony', 'request', 'session', 'args');
        $prop_keys = array_keys(get_object_vars($action));
        foreach ($properties as $name => $value) {
            if ($name[0] != '_' && !in_array($name, $ignore_keys) &&
                in_array($name, $prop_keys)) {
                $action->{$name} = $value;
            }
        }
    }

    /**
     * セッションに関するオプションを設定します。
     *
     * @access protected
     * @param object $session Moony_Sessionのインスタンス
     * @param array $session_options セッションのオプション設定
     */
    function _setSessionOptions(&$session, $session_options)
    {
        foreach ($session_options as $name => $value) {
            switch ($name) {
                case 'name':
                    $session->setName($value);
                    break;
                case 'id':
                    $session->setId($value);
                    break;
                case 'save_path':
                    $session->setSavePath($value);
                    break;
                case 'cache_expire':
                    $session->setCacheExpire($value);
                    break;
                case 'cache_limiter':
                    $session->setCacheLimiter($value);
                    break;
            }
        }
    }
}
?>
